package com.tzld.piaoquan.ad.engine.service.predict.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.ad.engine.commons.enums.AppTypeEnum;
import com.tzld.piaoquan.ad.engine.commons.redis.AlgorithmRedisHelper;
import com.tzld.piaoquan.ad.engine.commons.util.DateUtils;
import com.tzld.piaoquan.ad.engine.service.predict.config.RoiModelConfig;
import com.tzld.piaoquan.ad.engine.service.predict.constant.RuleRedisKeyConst;
import com.tzld.piaoquan.ad.engine.service.predict.PredictModelService;
import com.tzld.piaoquan.ad.engine.service.predict.config.AbConfig;
import com.tzld.piaoquan.ad.engine.service.predict.container.AbTestConfigContainer;
import com.tzld.piaoquan.ad.engine.service.predict.container.ThresholdModelContainer;
import com.tzld.piaoquan.ad.engine.service.predict.param.RoiThresholdPredictModelParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.RuleParamHelper;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdPredictModelParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.request.RoiPredictModelRequestParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.request.ThresholdPredictModelRequestParam;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PredictModelServiceImpl implements PredictModelService {

    @Value("${ad.own.experiment.tier:ab_test002}")
    private String adOwnExperimentTier;

    @Autowired
    private AbTestConfigContainer container;

    @Autowired
    AlgorithmRedisHelper redisHelper;

    @Autowired
    AbConfig abConfig;

    @Autowired
    RoiModelConfig roiModelConfig;

    public  Map<String,Object> adPredict(ThresholdPredictModelRequestParam requestParam){

        JSONObject object=requestParam.getAbExpInfo();
        //先判断是否开启实验 和是否不出广告时间 而后判断默认0-8
        Map<String,Object> result=new HashMap<>();
        int hourOfDay= DateUtils.getCurrentHour();
        if(container.containsCode(requestParam.getAbTestCode())
                &&
                container.inWithoutAdTime(requestParam.getAbTestCode(),hourOfDay)){
            result.put("ad_predict", 1);
            result.put("no_ad_strategy","no_ad_time");
            return result;
        }else  if(!container.containsCode(requestParam.getAbTestCode())&&(hourOfDay<8&&hourOfDay>=0)){
            result.put("ad_predict", 1);
            result.put("no_ad_strategy","no_ad_time");
            return result;
        }



        String[] abParamArr=abConfig.getAbParams(requestParam.getAbTestCode(),requestParam.getAbExpInfo());
        if(abParamArr==null){
            result.put("msg","abConfig_error");
            return result;
        }
        String abtestId=abParamArr[0];
        String abTestConfigTag=abParamArr[1];

        HashMap<String,Map<String,Object>> abConfigMap=abConfig.getAbConfigMap();
        Map<String,Object> abtestParam=abConfigMap.getOrDefault(abtestId+"-"+abTestConfigTag,null);
        if(abtestParam==null){
            result.put("msg","abConfig_error");
            return result;
        }
        //市-中文
        requestParam.setRegion(requestParam.getRegion().replace("省",""));
        requestParam.setCity(requestParam.getCity().replace("市",""));


        // Determine the group to which mid belongs
        String groupClassKey = (String) abtestParam.get("group_class_key");
        String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP + groupClassKey + ":" + requestParam.getMid();
        String midGroup = redisHelper.getString(midGroupKeyName);
        if (midGroup == null) {
            midGroup = "mean_group";
        }

        String[] noAdMidGroupList=(String[]) ((JSONArray) abtestParam.get("no_ad_mid_group_list")).toArray();

        boolean inNoAdGroup = false;
        for (String group : noAdMidGroupList) {
            if (group.equals(midGroup)) {
                inNoAdGroup = true;
                break;
            }
        }
        //不出广告组
        if (inNoAdGroup) {
            // User is in the no-ad group, no ad should be shown
            result.put("mid_group", midGroup);
            result.put("ad_predict", 1);
            result.put("no_ad_strategy","no_ad_mid_group_with_video");
            return result;
        }

        String[] noAdGroupWithVideoMapping=(String[]) ((JSONArray) abtestParam.get("no_ad_mid_group_list")).toArray();



        //设置信息
        ThresholdPredictModelParam modelParam=ThresholdPredictModelParam.builder()
                .build();
        BeanUtils.copyProperties(requestParam,modelParam);
        modelParam.setDate(new Date());
        modelParam.setAbtestId(abtestId);
        modelParam.setAbTestConfigTag(abTestConfigTag);
        modelParam.setAbtestParam(abtestParam);
        modelParam.setMidGroup(midGroup);
        Object thresholdMixFunc=abtestParam.getOrDefault("threshold_mix_func","basic");
        result= ThresholdModelContainer.
                getThresholdPredictModel(thresholdMixFunc.toString())
                .predict(modelParam);

        return result;
    }

    public Map<String,Object> adRecommendPredictByRoiModel(RoiPredictModelRequestParam requestParam){

        Map<String,Object> result=new HashMap<>();
        int hourOfDay= DateUtils.getCurrentHour();
        if(hourOfDay<8&&hourOfDay>=0){
            result.put("ad_predict", 1);
            result.put("no_ad_strategy","no_ad_time");
            return result;
        }
        RoiThresholdPredictModelParam modelParam=new RoiThresholdPredictModelParam();
        BeanUtils.copyProperties(requestParam,modelParam);
        modelParam.setEcpm(requestParam.getAds().get(0).getEcpm());
        modelParam.setAdId(requestParam.getAds().get(0).getAdId());
        modelParam.setAdType(requestParam.getAds().get(0).getAdType());

        String groupClassKey = (String)((Map<String,Object>)roiModelConfig.get(AppTypeEnum.valueOf(modelParam.getAppType()).name())).get("group_class_key");
        String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP + groupClassKey + ":" + modelParam.getMid();
        String midGroup = redisHelper.get(midGroupKeyName);
        if (midGroup == null) {
            midGroup = "mean_group";
        }
        modelParam.setMidGroup(midGroup);

        return ThresholdModelContainer.
                getThresholdPredictModel("roi")
                .predict(modelParam);
    }



}
